using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;

namespace binviz_0._1
{
    public partial class FrmEntropy : Form
    {
        private int globalOffset;
        private int globalFileSize;
        private int globalColumns;
        private FrmMain globalFrmMain;
        private double LOG2 = Math.Log10(2);

        public FrmEntropy()
        {
            InitializeComponent();
        }
        
        public void Plot(ref byte[] fileBufferArray, int fileSize, float[] frequency, int offset, FrmEncode frmEncode, FrmMain frmMain) {
            int columns = pictureBox1.Width;
            int rows = pictureBox1.Height;
            int position = offset;
            byte currentByte;
            globalFrmMain = frmMain;
            globalOffset = offset;
            globalFileSize = fileSize;
            globalColumns = columns;
            float H = 0.0F; //shannon entropy
            Bitmap b = new Bitmap(columns, rows, PixelFormat.Format24bppRgb);
            BitmapData bmd = b.LockBits(new Rectangle(0, 0, columns, rows), ImageLockMode.ReadWrite, b.PixelFormat); //PixelFormat.Format24bppRgb
       
            // The unsafe code block is necessary to operate directly on the image
            // memory vs. setpixel.  About 250-300x faster

         

            unsafe {
                for (int j = 0; j < rows; j++) {
                    byte* row = (byte*)bmd.Scan0 + (j * bmd.Stride); 
                    H = calcEntropy(ref fileBufferArray,offset+(j*640));
                    
                    columns = (int)(pictureBox1.Width * (H/8));
                    //Console.WriteLine("H=" + H);
                    if (globalFileSize > (offset+ (j* 640)))
                    {
                        row[columns * 3] = 255; 
                        row[columns * 3 + 1] = 255;
                        row[columns * 3 + 2] = 255; 
                    }
                }                
                b.UnlockBits(bmd);
                pictureBox1.Image = b;
             
               // Invert(b);
            } //unsafe
    
        }
                   
        public int lineWidth() {
            return(pictureBox1.Width);
        }

        //calculate shannon entropy for each row in the byte view
        //window size = 640, step size =640
        //note that if the user changes the byte view size (in this version
        //it is fixed) then these will need to be changed to the new width of the byte view
        public float calcEntropy(ref byte[] fileBufferArray,int offset){
            float H = 0.0F;
            byte currentByte;
            int[] windowByteCount = new int[256];
            float[] windowFrequency = new float[256];
          

            //init arrays, this might not be needed
            for (int i = 0; i <= 255; i++)
            {
                windowByteCount[i] = 0;
                windowFrequency[i] = 0;                
            }

            //count up bytes of each byte value
            for (int col = 0; col < 640; col++)
            {
              if (globalFileSize>(offset+col)){
                  windowByteCount[fileBufferArray[offset+col]]++;               
              }
            }

            //calculate frequency of each byte value
            for (int j = 0; j <= 255; j++)
            {
                windowFrequency[j] = (float)windowByteCount[j] / 640; //dividing int by int is integer division, must cast to float                   
                
            }

            //calculate shannon entropy
            //note that this can be included in the frequency calculation loop to speed things up
            for (int k = 0; k <= 255; k++)
            {
                if (windowFrequency[k] > 0.0)  //0 causes an error due to -infinity
                {
                    //H += windowFrequency[k]*(float)(Math.Log(windowFrequency[k],2));                  
                    //H += windowFrequency[k] * (float)(Math.Log10(windowFrequency[k]));                  
                    H += windowFrequency[k] * (float)(Math.Log10(windowFrequency[k])/LOG2);  
                }
            }
   
            return (H*-1);

        }

        private void pictureBox1_Click(object sender, EventArgs e)
        {

        }

        private void FrmEntropy_Load(object sender, EventArgs e)
        {

        }


        private void pictureBox1_MouseMove_1(object sender, MouseEventArgs e)
        {
            string locationString;
            double location = (e.X / (double)pictureBox1.Width) * 8.0;//* globalColumns + e.X + globalOffset);
            locationString = location.ToString();
            if (locationString.Length > 5)
            {
                locationString = locationString.Substring(0, 5);
            }
            if (location < globalFileSize) toolTip1.SetToolTip(this.pictureBox1, locationString);

        }

    
  

    }
}